;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Spiny Generator, by Mandew
;;
;; This sprite will shoot Spinies upwards at a constant interval.
;; You may customize the firing speed in these defines below.
;; With the Extra Bit, you can determine two different speeds.
;;
;; 2 - Timer set to Speed1 
;; 3 - Timer set to Speed2 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Defines
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Customizable defines

!Speed1 = #$3C ; Timer interval (x0.5) for Extra Bit set to 2
!Speed2 = #$1E ; Timer interval (x0.5) for Extra Bit set to 3

!SpinySpriteNumber = #$14 	; Sprite to spawn normally (vanilla; default spiny egg)
!SilverCoinSpriteNumber = #$21  ; Sprite to spawn if Silver P is active (vanilla; default Silver Coin)
!InitStatus = #$08 		; Status for the sprite to spawn in with.
	
	; If you want it to spawn a Koopa shell, set the Sprite Number to one of the regular Koopas,
	; and Init Status to #$09.

; Engine defines

!Layer1XPos = $1A
!Layer1YPos = $1C
!LevelIsVertical = $5B

!ShooterXPosLo = $179B
!ShooterXPosHi = $17A3
!ShooterYPosLo = $178B
!ShooterYPosHi = $1793
!ShooterTimer = $17AB
!ExtraBits = $1783

!GetNewIndex = $02A9DE
!InitSprTables = $07F7D2

!SpriteStatus = $14C8
!SilverPActive = $14AE

!SpriteNumber = $009E
!SpriteXPosLo = $00E4
!SpriteXPosHi = $14E0
!SpriteYPosLo = $00D8
!SpriteYPosHi = $14D4
!SpriteYSpeed = $AA
!YXPPCCCTtable = $15F6

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Wrapper
;
;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	STZ !ShooterTimer,x
	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR MainCode
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MainCode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpeedTable:
db !Speed1,!Speed2

DontShoot:
	RTS

MainCode:
	LDA !ShooterTimer,x
	BNE DontShoot
	LDY #$00
	LDA !ExtraBits,x
	AND #$40
	BEQ JustSpeed1
	INY
JustSpeed1:
	LDA SpeedTable,y
	STA !ShooterTimer,x

	LDA $5B
	AND #$01
	BNE VerticalLevel

	LDA !ShooterXPosLo,x
	STA $00
	LDA !ShooterXPosHi,x
	STA $01
	REP #$20
	LDA !Layer1XPos
	CMP $00
	BCC ToTheRight
ToTheLeft:
	SEC
	SBC #$0100
	CMP $00
	SEP #$20
	BCS DontShoot
	BRA DropSpinyEgg
ToTheRight:
	CLC
	ADC #$0200
	CMP $00
	SEP #$20
	BCC DontShoot
	BRA DropSpinyEgg

VerticalLevel:
	LDA !ShooterYPosLo,x
	STA $00
	LDA !ShooterYPosHi,x
	STA $01
	REP #$20
	LDA !Layer1YPos
	CMP $00
	BCC ToTheDown
ToTheUp:
	SEC
	SBC #$0100
	CMP $00
	SEP #$20
	BCS DontShoot
	BRA DropSpinyEgg
ToTheDown:
	CLC
	ADC #$0200
	CMP $00
	SEP #$20
	BCC DontShoot

;;;;;;;;;;;;;;;;;;;;;;;;;
; Drop that spiny
;;;;;;;;;;;;;;;;;;;;;;;;

DropSpinyEgg:
	JSL !GetNewIndex
	BMI ReturnYo
	LDA !InitStatus
	STA !SpriteStatus,Y
	LDA !SilverPActive
	CMP #$01
	LDA !SpinySpriteNumber
	BCC NotSilverCoin
	LDA !SilverCoinSpriteNumber
NotSilverCoin:
	STA !SpriteNumber,y

	LDA !ShooterXPosLo,X       
	STA !SpriteXPosLo,Y     
	LDA !ShooterXPosHi,X     
	STA !SpriteXPosHi,Y     
	LDA !ShooterYPosLo,X       
	STA !SpriteYPosLo,Y     
	LDA !ShooterYPosHi,X     
	STA !SpriteYPosHi,Y     
	PHX 
	TYX 

	JSL !InitSprTables 
	LDA #$D0
	STA !SpriteYSpeed,x
	
	CMP.B #$21                
	BNE NotSilverCoin2          
	LDA #$02                
	STA !YXPPCCCTtable,x
NotSilverCoin2:
	TXY                       
	PLX                       
ReturnYo:
	RTS 	